#include "SiteServer.h"

#include "Things/TemperatureSensor.h"

#if !defined(NO_STD)
#include <functional>
#include <memory>
#endif

namespace RoboidControl {

#pragma region Init

SiteServer::SiteServer(int port) : ParticipantUDPGeneric(port) {
  this->name = "Site Server";
  this->publishInterval = 0;

  //SetupUDP(port, ipAddress, 0);
}

#pragma endregion Init

#pragma region Update

void SiteServer::UpdateMyThings() {
  for (Thing* thing : this->things) {
    if (thing == nullptr)
      continue;

    thing->Update(true);

    if (this->isIsolated == false) {
      // Send to all other participants
#if defined(NO_STD)
      Participant** participants = Participant::registry.GetAll();
      for (int ix = 0; ix < Participant::registry.count; ix++) {
        Participant* participant = participants[ix];
#else
      for (Participant* participant : registry.GetAll()) {
#endif
        if (participant == nullptr || participant == this)
          continue;

        PoseMsg* poseMsg = new PoseMsg(this->networkId, thing);
        participant->Send(poseMsg);
        delete poseMsg;
        BinaryMsg* binaryMsg = new BinaryMsg(this->networkId, thing);
        participant->Send(binaryMsg);
        delete binaryMsg;
      }
    }
  }
}

#pragma endregion Update

#pragma region Receive

void SiteServer::Process(ParticipantSocket* sender, ParticipantMsg* msg) {
  if (msg->networkId != sender->networkId) {
    // std::cout << this->name << " received New Client -> " <<
    // sender->ipAddress
    //           << ":" << (int)sender->port << "\n";
    NetworkIdMsg* msg = new NetworkIdMsg(sender->networkId);
    sender->Send(msg);
    delete msg;
  }
}

void SiteServer::Process(ParticipantSocket* sender, NetworkIdMsg* msg) {}

void SiteServer::Process(ParticipantSocket* sender, ThingMsg* msg) {
  Thing* thing = sender->Get(msg->networkId, msg->thingId);
  if (thing == nullptr) {
    // new Thing(sender, (Thing::Type)msg->thingType, msg->thingId);
    // Thing::Reconstruct(sender, msg->thingType, msg->thingId);
    //thing = new Thing(msg->thingType, sender->root);
  }
  thing->id = msg->thingId;

  if (msg->parentId != 0) {
    thing->SetParent(Get(msg->networkId, msg->parentId));
    if (thing->IsRoot())
    // if (thing->GetParent() != nullptr)
#if defined(NO_STD)
      ;
#else
      std::cout << "Could not find parent [" << (int)msg->networkId << "/"
                << (int)msg->parentId << "]\n";
#endif
  } else
    thing->SetParent(Thing::LocalRoot());
}

#pragma endregion Receive

}  // namespace RoboidControl
